import BaseController from 'common/base.controller'
import { ZError } from 'common/ZError'
import { permission, router } from 'decorators/router'
import { AssetsInfo } from 'models/chain/AssetsInfo'

export default class ChainAssetsController extends BaseController {
  @permission(['assets:edit'])
  @router('post /api/chain/asset')
  async save(req) {
    let { _id } = req.params
    let user = req.user
    let record
    if (!_id) {
      record = new AssetsInfo(req.params)
      record.createdBy = user.id
    } else {
      record = await AssetsInfo.findById(_id)
      record.updateFromReq(req.params)
    }
    await record.save()
    return record.toJson()
  }

  @permission(['assets:read', 'form:edit', 'form:read'])
  @router('get /api/chain/asset/:key')
  async info(req) {
    const user = req.user
    const { key } = req.params
    const record = await AssetsInfo.findById(key)
    return record.toJson()
  }

  @permission(['assets:read', 'form:edit', 'form:read', 'flow:exec'])
  @router('post /api/chain/assets')
  async list(req) {
    const records = await AssetsInfo.pageQuery(req.params)
    return records
  }

  @permission('assets:delete')
  @router('delete /api/chain/asset')
  async delete(req) {
    let { ids } = req.params
    if (!ids) {
      throw new ZError(11, 'params mismatch')
    }
    let result = await AssetsInfo.deleteVirtual(ids)
    return result
  }
}
