import BaseController from '../../common/base.controller'
import { permission, router } from '../../decorators/router'
import { ZError } from '../../common/ZError'
import { Dept } from '../../models/admin/Dept'
import { ObjectId } from 'bson'

class DeptController extends BaseController {
  @permission(['dept:read', 'deptman:read'])
  @router('post /api/depts')
  async deptList(req) {
    return Dept.pageQuery(req.params)
  }

  @permission(['dept:read', 'deptman:read'])
  @router('get /api/dept/:id')
  async detail(req: any) {
    let { id } = req.params
    const record = await Dept.findById(id)
    if (!record) {
      throw new ZError(11, 'dept not found')
    }
    return record.toJson()
  }

  @permission('self:read')
  @router('get /api/mydept')
  async detailSelf(req: any) {
    let admin = req.user
    if (admin.level === 1) {
      throw new ZError(12, 'this api not for you')
    }
    if (!admin.department) {
      throw new ZError(13, 'you account has no dept bind')
    }
    const record = await Dept.findById(admin.department)
    if (!record) {
      throw new ZError(11, 'dept not found')
    }
    return record.toJson()
  }

  @permission(['dept:edit', 'deptman:edit'])
  @router('post /api/dept/save')
  async save(req: any) {
    const data = req.params
    if (!data._id) {
      data.createdBy = req.user.id
      data._id = new ObjectId()
    }
    let record = await Dept.insertOrUpdate({ _id: data._id }, data)
    return record.toJson()
  }

  @permission(['dept:delete', 'deptman:delete'])
  @router('post /api/dept/:id/delete')
  async delete(req: any) {
    let { ids } = req.params
    if (!ids) {
      throw new ZError(11, 'params mismatch')
    }
    await Dept.deleteVirtual(ids)
  }
}
