import BaseController from 'common/base.controller'
import { ZError } from 'common/ZError'
import { permission, router } from 'decorators/router'
import { FlowCategory } from 'models/flow/FlowCategory'

export default class FlowCategoryController extends BaseController {
  @permission('category:edit')
  @router('post /api/flow/category')
  async save(req) {
    let { _id } = req.params
    let user = req.user
    let record
    if (!_id) {
      record = new FlowCategory(req.params)
      record.createdBy = user.id
    } else {
      record = await FlowCategory.findById(_id)
      record.updateFromReq(req.params)
    }
    await record.save()
    return record.toJson()
  }

  @permission('category:read')
  @router('get /api/flow/category/:key')
  async info(req) {
    const user = req.user
    const { key } = req.params
    const record = await FlowCategory.findById(key)
    return record.toJson()
  }

  @permission(['category:read', 'definition:read', 'flow:exec'])
  @router('post /api/flow/categorys')
  async list(req) {
    const records = await FlowCategory.pageQuery(req.params)
    return records
  }

  @permission('category:delete')
  @router('delete /api/flow/category')
  async delete(req) {
    let { ids } = req.params
    if (!ids) {
      throw new ZError(11, 'params mismatch')
    }
    let result = await FlowCategory.deleteVirtual(ids)
    return result
  }
}
