import BaseController from 'common/base.controller'
import { ZError } from 'common/ZError'
import { permission, router } from 'decorators/router'
import { FlowDefinition } from 'models/flow/FlowDefinition'
import { FlowForm } from 'models/flow/FlowForm'
import { FlowNode } from 'models/flow/FlowNode'

export default class FlowDefinitionController extends BaseController {
  @permission('definition:edit')
  @router('post /api/flow/definition')
  async save(req) {
    let { _id } = req.params
    let user = req.user
    let record
    if (!_id) {
      record = new FlowDefinition(req.params)
      record.createdBy = user.id
    } else {
      record = await FlowDefinition.findById(_id)
      record.updateFromReq(req.params)
    }
    record.nodeCount = record.nodes.length
    await record.save()
    return record.toJson()
  }

  @permission('definition:read')
  @router('get /api/flow/definition/:key')
  async info(req) {
    const { key } = req.params
    const record = await FlowDefinition.findById(key)
    if (!record) {
      throw new ZError(10, 'record not found')
    }
    return record
  }

  @permission('definition:read')
  @router('post /api/flow/definition/:key')
  async infoDetail(req) {
    const { key } = req.params
    const record = await FlowDefinition.findById(key)
    if (!record) {
      throw new ZError(10, 'record not found')
    }
    let result: any = record.toJson()
    let nodes = []
    for (let sub of result.nodes) {
      let _sub: any = {
        _id: sub._id,
        node: sub.node,
        index: sub.index,
        executors: sub.executors,
      }
      let node = await FlowNode.findById(sub.node)
      if (node) {
        _sub.name = node.name
        _sub.type = node.type
        _sub.action = node.action
      }
      nodes.push(_sub)
    }
    result.nodes = nodes
    if (result.form) {
      let form = await FlowForm.findById(result.form)
      result.formData = form.toJson()
    }
    return result
  }

  @permission(['definition:read', 'flow:exec'])
  @router('post /api/flow/definitions')
  async list(req) {
    const records = await FlowDefinition.pageQuery(req.params)
    return records
  }

  @permission(['definition:read', 'flow:exec'])
  @router('post /api/flow/mydefinitions')
  async mylist(req) {
    let user = req.user
    let data = req.params
    let { start, limit, page } = data
    limit = +limit || 10
    start = +start || (+page - 1) * limit || 0
    // @ts-ignore
    let { opt, sort } = FlowDefinition.parseQueryParam(data)
    opt = Object.assign(opt, {
      'nodes.0.executors': user.id,
    })
    let records = await FlowDefinition.find(opt).sort(sort).skip(start).limit(limit)
    let total = await FlowDefinition.countDocuments(opt)
    return { records, total, start, limit }
  }

  @permission('definition:delete')
  @router('delete /api/flow/definition')
  async delete(req) {
    let { ids } = req.params
    if (!ids) {
      throw new ZError(11, 'params mismatch')
    }
    let result = await FlowDefinition.deleteVirtual(ids)
    return result
  }
}
