import BaseController from 'common/base.controller'
import { ZError } from 'common/ZError'
import { permission, router } from 'decorators/router'
import { FlowForm } from 'models/flow/FlowForm'

export default class FlowFormController extends BaseController {
  @permission('form:edit')
  @router('post /api/flow/form')
  async save(req) {
    let { _id } = req.params
    let user = req.user
    let record
    if (!_id) {
      record = new FlowForm(req.params)
      record.createdBy = user.id
    } else {
      record = await FlowForm.findById(_id)
      record.updateFromReq(req.params)
    }
    await record.save()
    return record.toJson()
  }

  @permission('form:read')
  @router('get /api/flow/form/:key')
  async info(req) {
    const user = req.user
    const { key } = req.params
    const record = await FlowForm.findById(key)
    return record.toJson()
  }

  @permission(['form:read', 'flow:exec'])
  @router('post /api/flow/forms')
  async list(req) {
    const records = await FlowForm.pageQuery(req.params)
    return records
  }

  @permission('form:delete')
  @router('delete /api/flow/form')
  async delete(req) {
    let { ids } = req.params
    if (!ids) {
      throw new ZError(11, 'params mismatch')
    }
    let result = await FlowForm.deleteVirtual(ids)
    return result
  }
}
