import BaseController from 'common/base.controller'
import { ZError } from 'common/ZError'
import { permission, router } from 'decorators/router'
import { FlowNode } from 'models/flow/FlowNode'
import { join } from 'path'

const fs = require('fs')

export default class FlowNodeController extends BaseController {
  @permission('node:edit')
  @router('post /api/flow/node')
  async save(req) {
    let { _id } = req.params
    let user = req.user
    let record
    if (!_id) {
      record = new FlowNode(req.params)
      record.createdBy = user.id
    } else {
      record = await FlowNode.findById(_id)
      record.updateFromReq(req.params)
    }
    await record.save()
    return record.toJson()
  }

  @permission('node:read')
  @router('get /api/flow/node/:key')
  async info(req) {
    const user = req.user
    const { key } = req.params
    const record = await FlowNode.findById(key)
    return record.toJson()
  }

  @permission('node:read')
  @router('post /api/flow/nodes')
  async list(req) {
    const records = await FlowNode.pageQuery(req.params)
    return records
  }

  @permission('node:delete')
  @router('delete /api/flow/node')
  async delete(req) {
    let { ids } = req.params
    if (!ids) {
      throw new ZError(11, 'params mismatch')
    }
    let result = await FlowNode.deleteVirtual(ids)
    return result
  }

  @permission('node:read')
  @router('get /api/flow/actions')
  async acountList(req) {
    const controllers = join(__dirname, '../../flowactions')
    let list: string[] = []
    fs.readdirSync(controllers)
      .filter((file: string) => ~file.search(/^[^.].*\.(ts|js)$/))
      .forEach((file: any) => {
        let f = file.replace('.js', '').replace('.ts', '')
        list.push(f)
      })
    return list
  }
}
