import BaseController from '../../common/base.controller'
import { permission, router } from '../../decorators/router'
import { AdminRole } from '../../models/admin/AdminRole'
import { ZError } from '../../common/ZError'
import { FastifyInstance } from 'fastify'

export default class RoleController extends BaseController {
  @permission('role:update')
  @router('post /api/roles')
  async saveRole(req) {
    const { name, permissions, key, level } = req.params
    const role = (await AdminRole.findOrCreate({ _id: key })).doc
    role.permissions = permissions
    role.name = name
    role.level = level
    await role.save()
    let rbac = (this as unknown as FastifyInstance).rbac
    rbac.remove(role.id)
    for (let data of permissions) {
      if (data === '*') {
        rbac.add(role.id, data, '*')
      } else {
        const opers = data.split(':')
        rbac.add(role.id, opers[0], opers[1])
      }
    }
    rbac.add(role.id, 'self', '*')
    return role.toJson()
  }

  @permission(['role:read', 'shopadmin:read'])
  // @role('sysadmin')
  @router('get /api/roles')
  async roles(req) {
    const user = req.user
    const queryData: any = {}
    if (!user.isSysAdmin()) {
      queryData.show = true
    }
    const { level } = req.params
    if (level != undefined) {
      queryData.level = { $gte: level }
    }
    const records = await AdminRole.find(queryData)
    return records.map(o => o.toJson())
  }

  @permission('role:delete')
  @router('delete /api/roles/:key')
  async deleteRole(req) {
    const { key } = req.params
    if (!key) {
      throw new ZError(10, 'record not found')
    }
    const result = await AdminRole.deleteOne({ _id: key })
    let rbac = (this as unknown as FastifyInstance).rbac
    rbac.remove(key)
    return { count: result.deletedCount }
  }
}
