import { ZError } from 'common/ZError'
import { FlowInstance } from 'models/flow/FlowInstance'
import { ChainSvr } from 'services/ChainSvr'
import { IAction } from './base/IAction'
/**
 * let data = {
      taskId: '10009',
      type: 5,
      data: [{ address: '0xd45A464a2412A2f83498d13635698a041b9dBe9b', tokenIds: ['3'] }],
    }
 */
export default class implements IAction {
  async execute(id: string) {
    const maxCount = parseInt(process.env.CHAIN_MAX_BATCH_COUNT)
    let record = await FlowInstance.findById(id)
    if (!record) {
      throw new ZError(10, 'no record found')
    }
    let map: Map<string, string[]> = new Map()
    for (let sub of record.flowData.records) {
      let key = sub.address
      let data: string[]
      if (!map.has(key)) {
        data = []
        map.set(key, data)
      }
      data = map.get(key)
      data.push(sub.tokenid)
    }
    let datas = []
    for (let [key, value] of map.entries()) {
      while (value.length > 0) {
        let v = {
          address: key,
          tokenIds: value.splice(0, maxCount),
        }
        datas.push(v)
      }
    }
    let reqData = {
      taskId: id,
      type: 5,
      data: datas,
    }
    return new ChainSvr().addTask(reqData)
  }
}
