import { ChainSvr } from 'services/ChainSvr'
import { IAction } from './base/IAction'
import { FlowInstance, FlowStatus, FlowTask } from 'models/flow/FlowInstance'
import { ZError } from 'common/ZError'

interface IMapData {
  address: string
  to: string
  tokenIds: string[]
  configIds: string[]
}
/**
 * let data = {
      taskId: '1',
      type: 2,
      data: [
        {
          address: '0xd45A464a2412A2f83498d13635698a041b9dBe9b',
          to: '0x90F8bf6A479f320ead074411a4B0e7944Ea8c9C1',
          tokenIds: ['3'],
          configIds: ['1'],
        },
      ],
    }
 */
export default class implements IAction {
  constructor() {
    return this
  }
  async execute(id: string) {
    const maxCount = parseInt(process.env.CHAIN_MAX_BATCH_COUNT)
    let record = await FlowInstance.findById(id)
    if (!record) {
      throw new ZError(10, 'no record found')
    }
    let map: Map<string, IMapData> = new Map()
    for (let sub of record.flowData.records) {
      let key = sub.address + sub.to
      let data: IMapData
      if (!map.has(key)) {
        data = {
          address: sub.address,
          to: sub.to,
          tokenIds: [],
          configIds: [],
        }
        map.set(key, data)
      }
      data = map.get(key)
      data.tokenIds.push(sub.tokenid)
      data.configIds.push(sub.configid)
    }
    let datas = []
    for (let [key, value] of map.entries()) {
      while (value.tokenIds.length > 0) {
        let v = {
          address: value.address,
          to: value.to,
          tokenIds: value.tokenIds.splice(0, maxCount),
          configIds: value.configIds.splice(0, maxCount),
        }
        datas.push(v)
      }
    }
    let reqData = {
      taskId: id,
      type: 2,
      data: datas,
    }
    console.log(JSON.stringify(reqData))
    return new ChainSvr().addTask(reqData)
  }
}
