import { getModelForClass, modelOptions, prop } from '@typegoose/typegoose'
import { dbconn } from '../decorators/dbconn'
import { Base, TimeStamps } from '@typegoose/typegoose/lib/defaultClasses'

@modelOptions({ schemaOptions: { _id: false } })
export class PlatInfo {
  @prop()
  public platId: string
  @prop()
  public platAppId: string
  @prop()
  public platSecret: string
  @prop()
  public platParam1: string
  @prop()
  public platParam2: string
  @prop({ default: 0 })
  public status: number
  @prop()
  public platToken: string
  @prop()
  public comment?: string
  @prop({ default: new Date() })
  public add_time: Date
}
interface AppClass extends TimeStamps {}
@dbconn()
@modelOptions({ schemaOptions: { collection: 'app_info', timestamps: true } })
class AppClass extends Base<string> {
  @prop()
  public _id: string
  @prop()
  public name: string
  @prop()
  public icon: string
  @prop({ type: () => [PlatInfo], default: [] })
  public plats?: PlatInfo[]
  @prop()
  public comment?: string

  /**
   * 根据platId获取平台配置信息
   * @param platId
   */
  public getPlatByPlatId(platId: string): PlatInfo {
    if (!this.plats) {
      return null
    } else {
      for (let plat of this.plats) {
        if (plat.platId === platId) {
          return plat
        }
      }
      return null
    }
  }

  /**
   * 根据AppId获取平台配置信息
   * @param appId
   */
  public getPlatByAppId(appId: string): PlatInfo {
    if (!this.plats) {
      return null
    } else {
      for (let plat of this.plats) {
        if (plat.platAppId === appId) {
          return plat
        }
      }
      return null
    }
  }

  /**
   * 根据appId查找AppInfo
   * @param appId
   */
  public static async findByAppId(appId: string) {
    return AppInfo.findOne({ 'plats.platAppId': appId })
  }

  public static async findPlat(appId: string, platId: string) {
    let appInfo = await AppInfo.findById(appId)
    if (!appInfo) {
      return null
    }
    let platInfo: PlatInfo = appInfo.getPlatByPlatId(platId)
    if (!appInfo.plats || !platInfo) {
      return null
    }
    return platInfo
  }
}

export const AppInfo = getModelForClass(AppClass, { existingConnection: AppClass['db'] })
