import { getModelForClass, modelOptions, prop } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'
import { BaseModule } from '../Base'

@dbconn()
@modelOptions({
  schemaOptions: {
    collection: 'admin_permission',
    _id: false,
    timestamps: true,
  },
})
export class AdminPermissionClass extends BaseModule {
  @prop()
  public _id: string

  @prop()
  public name: string

  @prop({ type: () => [String] })
  public actions: string[]

  @prop()
  public comment?: string

  @prop({ default: 999 })
  public sort: number

  @prop({ default: true })
  public show: boolean
  /**
   * 权限选择时, 需要的用户等级
   * 用户等级需要小于或等于该值才可显示
   * @type {number}
   */
  @prop()
  public level: number

  public toJson() {
    return {
      id: this._id,
      label: this.name,
      children: this.actions.map(o => {
        return { id: `${this._id}:${o}`, label: o }
      }),
      comment: this.comment,
      level: this.level,
    }
  }
}

export const AdminPermission = getModelForClass(AdminPermissionClass, { existingConnection: AdminPermissionClass.db })
