import { getModelForClass, modelOptions, prop } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'
import { BaseModule } from '../Base'

@dbconn()
@modelOptions({ schemaOptions: { collection: 'admin_role', _id: false, timestamps: true } })
export class AdminRoleClass extends BaseModule {
  @prop()
  public _id: string
  @prop()
  public name: string
  @prop({ default: true })
  public show: boolean
  @prop({ type: () => [String] })
  public permissions: string[]
  @prop({ type: () => [String] })
  public inherits?: string[]
  /**
   * 权限选择时, 需要的用户等级
   * 用户等级需要小于或等于该值才可显示
   * @type {number}
   */
  @prop()
  public level: number
  @prop()
  public comment?: string

  public static async allRbacJson() {
    let records = await AdminRole.find()
    let result = {}
    for (let record of records) {
      record.permissions.push('self:*')
      result[record._id] = {
        can: record.permissions,
        inherits: record.inherits,
      }
    }
    return result
  }
  public toJson() {
    return {
      key: this._id,
      name: this.name,
      permissions: this.permissions,
      comment: this.comment,
      level: this.level,
    }
  }
}

export const AdminRole = getModelForClass(AdminRoleClass, { existingConnection: AdminRoleClass.db })
