import { getModelForClass, index, modelOptions, pre, prop } from '@typegoose/typegoose'
import { dbconn } from '../../decorators/dbconn'
import { noJson } from '../../decorators/nojson'
import { BaseModule } from '../Base'
import { IDCounter } from '../IDCounter'

@dbconn()
@index({ partnerId: 1 }, { unique: false })
@modelOptions({ schemaOptions: { collection: 'dept', timestamps: true } })
@pre<DeptClass>('save', async function () {
  this.newRecord = false
  if (!this.numid) {
    let counter = await IDCounter.nextCount()
    this.numid = counter.seq
  }
})
export class DeptClass extends BaseModule {
  @prop()
  public numid: number

  @prop({ required: true })
  public name!: string
  /**
   * 显示名
   * @type {string}
   */
  @prop()
  public showName: string

  /**
   * 是否删除
   * @type {boolean}
   */
  @noJson()
  @prop({ default: false })
  public deleted: boolean
  @noJson()
  @prop()
  public deleteTime: Date

  /**
   * 创建人
   * @type {string}
   */
  @prop()
  public createdBy: string
  /**
   * 一些系统级的部门不需要显示
   * @type {boolean}
   */
  @noJson()
  @prop({ default: true })
  public show: boolean

  /**
   * 是否为新记录, 只用于判断insertOrUpdate返回的记录是否为新纪录
   * @type {boolean}
   */
  @prop({ default: true })
  public newRecord: boolean

  public static parseQueryParam(params) {
    let options: any = {
      opt: { deleted: false, show: true, publish: true },
      matchKey: 'name',
    }
    let { opt, sort } = super.parseQueryParam(params, options)
    let { publish } = params
    if (publish !== undefined) {
      if (typeof publish === 'boolean') {
        opt.publish = publish
      } else {
        if (publish === 'all') {
          delete opt.publish
        } else if (publish === 'true') {
          opt.publish = true
        } else if (publish === 'false') {
          opt.publish = false
        }
      }
    }

    return { opt, sort }
  }

  public static async fetchByID(id: string) {
    return Dept.findById(id)
  }

  public static findByPartnerId(pid: string) {
    return Dept.findOne({ partnerId: pid })
  }

  public toPartnerJson() {
    const exportKeys = ['_id', 'sid', 'partnerId', 'name', 'showName', 'logo', 'createdAt', 'publish']
    let result: any = {}
    for (let key of exportKeys) {
      result[key] = this[key]
    }
    return result
  }
}

export const Dept = getModelForClass(DeptClass, { existingConnection: DeptClass.db })
