import { getModelForClass, index, modelOptions, prop } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'
import { noJson } from 'decorators/nojson'
import { BaseModule } from '../Base'

export enum AssetsType {
  ERC20 = 1,
  ERC721 = 2,
  ERC1155 = 3,
  OTHER = 9,
}
/**
 * 一个流程节点
 */
@dbconn()
@index({ address: 1 }, { unique: true })
@modelOptions({ schemaOptions: { collection: 'chain_assets_info', timestamps: true } })
class AssetsInfoClass extends BaseModule {
  @prop({ required: true })
  public name!: string

  @prop({ enum: AssetsType, required: true })
  public type!: AssetsType

  @prop({ required: true })
  public address!: string

  @prop()
  public chain: string

  @prop()
  public comment: string

  @prop({ required: true, default: 999 })
  public sort: number
  /**
   * 是否删除
   * @type {boolean}
   */
  @noJson()
  @prop({ default: false })
  public deleted: boolean
  @noJson()
  @prop()
  public deleteTime: Date

  /**
   * 创建人
   * @type {string}
   */
  @prop()
  public createdBy: string
}

export const AssetsInfo = getModelForClass(AssetsInfoClass, { existingConnection: AssetsInfoClass['db'] })
