import { getModelForClass, modelOptions, prop, ReturnModelType } from '@typegoose/typegoose'
import { dbconn } from '../../decorators/dbconn'
import { BaseModule } from '../Base'
import { noJson } from '../../decorators/nojson'
import { Base } from '@typegoose/typegoose/lib/defaultClasses'

export class GameVersion extends Base {
  /**
   * 游戏类型
   * 0: 网页
   * 1: 微信小游戏
   * @type {number}
   */
  @prop()
  public type: number

  @prop()
  public url: string

  @prop()
  public appid: string

  @prop()
  public appsecret: string

  @prop()
  public image: string

  @prop()
  public name: string
}

@dbconn()
@modelOptions({ schemaOptions: { collection: 'game_info', timestamps: true } })
class GameClass extends BaseModule {
  @prop({ required: true })
  public name!: string

  @prop({ type: () => [GameVersion] })
  public versions: GameVersion[]

  /**
   * 是否删除
   * @type {boolean}
   */
  @noJson()
  @prop({ default: false })
  public deleted: boolean
  @noJson()
  @prop()
  public deleteTime: Date

  /**
   * 创建人
   * @type {string}
   */
  @prop()
  public createdBy: string

  public static parseQueryParam(params) {
    let options: any = {
      matchKey: 'name',
    }
    let { opt, sort } = super.parseQueryParam(params, options)
    let { hasVersion } = params
    if (hasVersion) {
      Object.assign(opt, { 'versions.0': { $exists: true } })
    }
    return { opt, sort }
  }

  public static async findAvaOne(this: ReturnModelType<typeof GameClass>) {
    return this.findOne({ deleted: false, 'versions.0': { $exists: true } }).exec()
  }

  public static async fetchVersionInfo(gameId: string, version: string) {
    let game = await Game.findById(gameId)
    if (!game) {
      return null
    }
    return game.versions.find(o => o._id.toHexString() === version)
  }
}

export const Game = getModelForClass(GameClass, { existingConnection: GameClass.db })
