import { dbconn } from '../../decorators/dbconn'
import { getModelForClass, modelOptions, pre, prop } from '@typegoose/typegoose'
import { BaseModule } from '../Base'
import { INIT_KEY_NUM, MongoTool } from '../../services/MongoTool'

@dbconn()
@modelOptions({ schemaOptions: { collection: 'game_items', timestamps: true, _id: false } })
@pre<GameItemClass>('save', async function () {
  if (!this._id) {
    let sid = 0
    while (!sid) {
      sid = new MongoTool().getUniqueKey('gameitem')
    }
    this._id = sid + ''
  }
})
class GameItemClass extends BaseModule {
  @prop()
  public _id: string
  @prop()
  public shop: string

  @prop()
  public name: string

  @prop({ default: 0 })
  public type: number

  public static async maxNumKey() {
    const records = await GameItem.find().sort({ _id: -1 }).limit(1)
    let result = INIT_KEY_NUM
    if (records && records.length > 0) {
      result = Number(records[0]._id) || result
    }
    return result
  }
}

export const GameItem = getModelForClass(GameItemClass, { existingConnection: GameItemClass.db })
