import { dbconn } from '../../decorators/dbconn'
import { getModelForClass, modelOptions, prop, ReturnModelType } from '@typegoose/typegoose'
import { Severity } from '@typegoose/typegoose/lib/internal/constants'
import { BaseModule } from '../Base'
import { noJson } from '../../decorators/nojson'
import { SysMail } from './SysMail'

@dbconn()
@modelOptions({
  schemaOptions: { collection: 'sys_announce', timestamps: true },
  options: { allowMixed: Severity.ALLOW },
})
export class SysAnnounceClass extends BaseModule {
  @prop()
  public shop: string

  @prop()
  public title: string

  @prop()
  public content: string

  /**
   * 类型
   * @type {number} 0: 普通 1: 店铺群发 2: 全体
   */
  @prop()
  public type: number

  @noJson()
  @prop({ default: false })
  public publish: boolean

  @noJson()
  @prop({ type: () => [String] })
  public accounts: string[]

  @noJson()
  @prop({ type: () => [String] })
  public shops: string[]

  /**
   * 是否删除
   * @type {boolean}
   */
  @noJson()
  @prop({ default: false })
  public deleted: boolean
  @noJson()
  @prop()
  public deleteTime: Date

  /**
   * 创建人
   * @type {string}
   */
  @noJson()
  @prop()
  public createdBy: string

  @prop()
  public sendTime: number

  @prop()
  public endTime: number

  public static async findRecords(
    this: ReturnModelType<typeof SysAnnounceClass>,
    accountId: string,
    shop: string,
    excludes: string[],
  ) {
    let now = Date.now()
    let mails0 = await this.find({
      accounts: accountId,
      type: 0,
      deleted: false,
      publish: true,
      sendTime: { $lte: now },
      endTime: { $gte: now },
      _id: { $nin: excludes },
    })
    let mails1 = await this.find({
      shops: shop,
      type: 1,
      deleted: false,
      publish: true,
      sendTime: { $lte: now },
      endTime: { $gte: now },
      _id: { $nin: excludes },
    })
    let mails2 = await this.find({
      type: 2,
      deleted: false,
      publish: true,
      sendTime: { $lte: now },
      endTime: { $gte: now },
      _id: { $nin: excludes },
    })
    return mails0.concat(mails1).concat(mails2)
  }
}

export const SysAnnounce = getModelForClass(SysAnnounceClass, { existingConnection: SysAnnounceClass.db })
