import { dbconn } from '../../decorators/dbconn'
import { getModelForClass, index, modelOptions, prop } from '@typegoose/typegoose'
import { Severity } from '@typegoose/typegoose/lib/internal/constants'
import { BaseModule } from '../Base'
import { noJson } from '../../decorators/nojson'

export class MailItemClass {
  @prop()
  public itemId: string

  @prop()
  public name: string

  @prop()
  public count: number
}

@dbconn()
@index({ accounts: 1, deleted: 1, type: 1, sendTime: 1, endTime: 1 }, { unique: false })
@index({ shops: 1, deleted: 1, type: 1, sendTime: 1, endTime: 1 }, { unique: false })
@modelOptions({
  schemaOptions: { collection: 'sys_mail', timestamps: true },
  options: { allowMixed: Severity.ALLOW },
})
export class SysMailClass extends BaseModule {
  @prop()
  public sender: string

  @prop()
  public senderShop: string

  @prop()
  public title: string

  @prop()
  public content: string

  /**
   * 邮件类型
   * @type {number} 0: 普通邮件 1: 店铺群发
   */
  @prop({ default: 0 })
  public type: number

  @prop({ type: () => [String] })
  public accounts: string[]

  @prop({ type: () => [String] })
  public shops: string[]

  @prop({ type: () => [MailItemClass] })
  public items: MailItemClass[]

  /**
   * 是否删除
   * @type {boolean}
   */
  @noJson()
  @prop({ default: false })
  public deleted: boolean
  @noJson()
  @prop()
  public deleteTime: Date

  /**
   * 创建人
   * @type {string}
   */
  @prop()
  public createdBy: string

  @prop()
  public sendTime: number

  @prop()
  public endTime: number

  public static async findMail(accountId: string, shop: string, excludes: string[]) {
    let now = Date.now()
    let mails0 = await SysMail.find({
      accounts: accountId,
      type: 0,
      deleted: false,
      sendTime: { $lte: now },
      endTime: { $gte: now },
      _id: { $nin: excludes },
    })
    let mails1 = await SysMail.find({
      shops: shop,
      type: 1,
      deleted: false,
      sendTime: { $lte: now },
      endTime: { $gte: now },
      _id: { $nin: excludes },
    })
    return mails0.concat(mails1)
  }
}
export const SysMail = getModelForClass(SysMailClass, { existingConnection: SysMailClass.db })
