import { getModelForClass, index, modelOptions, pre, prop } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'
import { noJson } from 'decorators/nojson'
import { BaseModule } from '../Base'

@modelOptions({ schemaOptions: {} })
export class NodeInfo {
  @prop({ required: true })
  public node!: string
  @prop({ required: true, default: 999 })
  public index: number
  @prop({ type: () => [String] })
  public executors: string[]
}
/**
 * 一个流程定义
 */
@dbconn()
@modelOptions({ schemaOptions: { collection: 'flow_definition', timestamps: true } })
class FlowDefinitionClass extends BaseModule {
  @prop({ required: true })
  public name!: string
  @prop({ required: true })
  public category: string
  @prop({ required: true, default: 0 })
  public nodeCount: number

  @prop({ type: () => [NodeInfo] })
  public nodes: NodeInfo[]
  // 表单数据
  @prop()
  public form?: string

  /**
   * 是否删除
   * @type {boolean}
   */
  @noJson()
  @prop({ default: false })
  public deleted: boolean
  @noJson()
  @prop()
  public deleteTime: Date

  /**
   * 创建人
   * @type {string}
   */
  @prop()
  public createdBy: string
}

export const FlowDefinition = getModelForClass(FlowDefinitionClass, { existingConnection: FlowDefinitionClass['db'] })
