import { getModelForClass, index, modelOptions, pre, prop } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'
import { noJson } from 'decorators/nojson'
import { BaseModule } from '../Base'

export enum DataTypeEnum {
  string = 0,
  number = 1,
  datetime = 2,
  date = 3,
  time = 4,
  address = 5,
  other = 9,
}
/**
 * 一个数据项
 */
@modelOptions({ schemaOptions: { _id: false } })
export class InputInfo {
  @prop()
  public keyid: string
  @prop()
  public name: string
  // 默认值
  @prop()
  public defaultVal: string
  // 用于select类型的可选项
  @prop({ type: () => [String], default: [] })
  public values: string[]
  @prop({ required: true, default: DataTypeEnum.string })
  public dataType: DataTypeEnum
  @prop({ required: true, default: 999 })
  public index: number
  @prop({ required: true, default: false })
  public unique: boolean
  // 该值只针对address的数据, 关联chain/AssetsInfo
  @prop()
  public addressType: string[]

  public toJson() {
    return {
      keyid: this.keyid,
      name: this.name,
      defaultVal: this.defaultVal,
      values: this.values,
      dataType: this.dataType,
      index: this.index,
      unique: this.unique,
      addressType: this.addressType,
    }
  }
}
/**
 * 流程使用的表单数据定义
 */
@dbconn()
@modelOptions({ schemaOptions: { collection: 'flow_form', timestamps: true } })
class FlowFormClass extends BaseModule {
  @prop({ required: true })
  public name!: string
  @prop({ type: () => [InputInfo] })
  public nodes: InputInfo[]

  /**
   * 是否删除
   * @type {boolean}
   */
  @noJson()
  @prop({ default: 0 })
  public deleted: number
  @noJson()
  @prop()
  public deleteTime: Date

  /**
   * 创建人
   * @type {string}
   */
  @prop()
  public createdBy: string
}

export const FlowForm = getModelForClass(FlowFormClass, { existingConnection: FlowFormClass['db'] })
