import { getModelForClass, modelOptions, prop } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'
import { noJson } from 'decorators/nojson'
import { BaseModule } from '../Base'

export enum NodeType {
  start = 0,
  normal = 1, // 普通的审批节点
  exec = 2, // 自动处理业务逻辑的节点
  end = 9,
}
/**
 * 一个流程节点
 */
@dbconn()
@modelOptions({ schemaOptions: { collection: 'flow_node', timestamps: true } })
class FlowNodeClass extends BaseModule {
  @prop({ required: true })
  public name!: string
  @prop({ enum: NodeType, required: true })
  public type: NodeType
  // 关联的特定的js类, 须实现特定的接口
  @prop()
  public action: string

  @prop({ required: true, default: 999 })
  public sort: number
  /**
   * 是否删除
   * @type {boolean}
   */
  @noJson()
  @prop({ default: false })
  public deleted: boolean
  @noJson()
  @prop()
  public deleteTime: Date

  /**
   * 创建人
   * @type {string}
   */
  @prop()
  public createdBy: string
}

export const FlowNode = getModelForClass(FlowNodeClass, { existingConnection: FlowNodeClass['db'] })
