import { IPlat, IPlatUser } from './IPlat'

import AlipaySdk from 'alipay-sdk'

/**
 * 支付宝 生活号/H5 内获取用户信息
 * https://opendocs.alipay.com/open/284/h5
 * 1. 页面加载js https://3g.163.com/dy/article/FMR28QU2051100B9.html
 * 2. 调用
 * ap.getAuthCode ({
      appId :  '${appId}' ,
      scopes : [ 'auth_user' ],
   },  function ( res ){
      ap.alert ( JSON.stringify ( res ));
   });
 * 3. 拿到res.authCode后调用登陆接口
 *
 */
class AliplayApi {
  alipaySdk: AlipaySdk
  constructor({ platAppId, platSecret }) {
    let AlipaySdkConfig = {
      appId: platAppId,
      privateKey: platSecret,
    }

    this.alipaySdk = new AlipaySdk(AlipaySdkConfig)
  }
  /**
   * 获取accToken
   * @param {string} code
   */
  async accToken(code) {
    console.log('code', code)
    try {
      let params = {
        grantType: 'authorization_code',
        code,
      }
      let options = {}
      let r1 = await this.alipaySdk.exec('alipay.system.oauth.token', params, options)
      return r1
    } catch (e) {
      console.log(e)
    }
  }

  /**
   * 获取用户信息
   * @param accessToken
   */
  async userInfo(accessToken) {
    try {
      let params = {
        auth_token: accessToken,
      }
      let options = {}
      let r1 = await this.alipaySdk.exec('alipay.user.info.share', params, options)
      return r1
    } catch (e) {
      console.log(e)
    }
  }
}

export class PlatAliPayPage implements IPlat {
  platAppId: string
  platSecret: string
  constructor({ platAppId, platSecret }) {
    this.platAppId = platAppId
    this.platSecret = platSecret
  }
  async codeToSession({ code, needUInfo }): Promise<IPlatUser> {
    let client = new AliplayApi({ platAppId: this.platAppId, platSecret: this.platSecret })
    let data: IPlatUser = { openid: '', unionid: '', sessionKey: '' }
    const result = await client.accToken(code)
    const accessToken = result['accessToken']
    const uinfo = await client.userInfo(accessToken)
    data.unionid = result['alipayUserId']
    data.openid = uinfo['userId']
    data.nickname = uinfo['nickName']
    data.headimgurl = uinfo['avatar']
    data.province = uinfo['province']
    data.city = uinfo['city']
    data.sex = uinfo['gender']
    return Promise.resolve(data)
  }

  /**
   * 拼接用户授权的 URL
   * @param scope auth_user（获取用户信息、网站支付宝登录）、auth_base（用户信息授权）、auth_ecard（商户会员卡）、auth_invoice_info（支付宝闪电开票）、auth_puc_charge（生活缴费）五个值;多个scope时用”,”分隔
   * @param cbUrl
   * @param state 商户自定义参数, 最大长度100, 用户授权后，重定向到redirect_uri时会原样回传给商户
   */
  getAuthorizeURL(scope: string, cbUrl: string, state: string) {
    return `https://openauth.alipay.com/oauth2/publicAppAuthorize.htm?app_id=${
      this.platAppId
    }&scope=${scope}&redirect_uri=${encodeURIComponent(cbUrl)}&state=${state}`
  }
}
