import { IPlat, IPlatData, IPlatUser } from './IPlat'
import { PlatInfo } from '../models/AppInfo'
import { hmac } from '../utils/security.util'

const http = require('http')
const https = require('https')
const url = require('url')
const urlencode = require('urlencode')
//https://iopen.game.oppomobile.com/sdkopen/user/fileIdInfo?fileId=ssoid&token=
const accountAuthUrl = 'https://iopen.game.oppomobile.com/sdkopen/user/fileIdInfo'

let HttpGet_HeaderJson = function (url_str, data, callback) {
  let parse_u = url.parse(url_str)
  let isHttp = parse_u.protocol == 'http:'
  let options = {
    host: parse_u.hostname,
    port: parse_u.port || (isHttp ? 80 : 443),
    path: parse_u.path,
    method: 'get',
    headers: {
      param: data.param,
      oauthSignature: data.oauthSignature,
    },
  }
  ;(isHttp ? http : https)
    .get(options, function (response) {
      let res_data = ''
      response.on('data', function (chunk) {
        res_data += chunk
      })
      response.on('end', function () {
        if (callback) callback(undefined, res_data)
      })
    })
    .on('error', function (e) {
      callback(e.message)
    })
}

export class PlatOppoNative implements IPlat {
  platAppId: string
  appsecret: string
  appkey: string

  constructor(platInfo: PlatInfo) {
    this.platAppId = platInfo.platAppId
    this.appsecret = platInfo.platSecret
    this.appkey = platInfo.platParam1
  }

  codeToSession(data: IPlatData): Promise<IPlatUser> {
    let request_serverUrl = `${accountAuthUrl}?fileId=${urlencode.encode(data.code, 'utf8')}&token=${urlencode.encode(
      data.token,
      'utf8',
    )}`
    let nonce = parseInt(Math.random() * (9999999999 - 1000000000 + 1000000000) + '')
    let ts = parseInt(new Date().getTime() / 1000 + '')
    console.log('---oppo-request_serverUrl=' + request_serverUrl)
    let json = {
      oauthConsumerKey: this.appkey,
      oauthToken: urlencode.encode(data.token, 'utf8'),
      oauthSignatureMethod: 'HMAC-SHA1',
      oauthTimestamp: ts,
      oauthNonce: nonce,
      oauthVersion: '1.0',
    }
    let baseStr = ''
    for (let key in json) {
      baseStr += key + '=' + json[key] + '&'
    }
    console.log('---oppo-baseStr=' + baseStr)
    // let beseString = new Buffer(baseStr, 'utf8').toString();
    baseStr = baseStr.replace(/\+/g, '%2B')

    let sign = urlencode.encode(hmac(baseStr, this.appsecret + '&', 'base64'), 'utf8').toString()

    console.log('---oppo-sign=' + sign)
    let authString = {
      param: baseStr,
      oauthSignature: sign,
    }
    console.log('---authString=%s', JSON.stringify(authString))
    return new Promise((resolve, reject) => {
      HttpGet_HeaderJson(request_serverUrl, authString, function (err, ret) {
        try {
          if (err) {
            console.log('Oppo 帐号get请求异常:' + err)
            return reject && reject(err)
          }

          let retData = JSON.parse(ret)
          console.log('Oppo 帐号登陆回调信息：' + ret)
          if (retData.resultCode != 200) {
            console.log('Oppo 帐号登陆失败:' + retData.resultMsg)
            return reject && reject(retData.resultMsg)
          }

          if (!retData.ssoid || retData.ssoid !== data.code) {
            console.log('Oppo 帐号登陆失败 access_token_secret=[%s] retData.ssoid=[%s]', data.code, retData.ssoid)
            return reject && reject('登陆失败')
          }

          console.log('Oppo 帐号登陆成功:' + retData)
          resolve && resolve({ openid: retData.ssoid, nickname: retData.userName, sessionKey: retData.loginToken })
        } catch (err) {
          console.log('Oppo 登陆异常' + err)
          return reject && reject('登陆异常')
        }
      })
    })
  }

  preOrder(platInfo: PlatInfo, data: any): any {}
}
