import { IPlat, IPlatUser } from './IPlat'
import { NetClient } from '../net/NetClient'
import { generateKeyValStr } from '../utils/string.util'
import { md5 } from '../utils/security.util'
import { PlatInfo } from '../models/AppInfo'
import querystring from 'querystring'

const netClient = new NetClient()

const accountAuthUrl = 'https://joint-account.vivo.com.cn/cp/user/auth'

export class PlatVivoNative implements IPlat {
  platAppId: string
  platCpid: string
  authKey: string
  payKey: string

  constructor(platInfo: PlatInfo) {
    this.platAppId = platInfo.platAppId
    this.platCpid = platInfo.platSecret
    this.authKey = platInfo.platParam1
    this.payKey = platInfo.platParam2
  }

  static generateVivoSign(data: {}, key: string) {
    let signStr = generateKeyValStr(data, true)
    let keyHash = md5(key)
    signStr += `&${keyHash.toLowerCase()}`
    return md5(signStr)
  }

  codeToSession({ code }): Promise<IPlatUser> {
    const params = querystring.stringify({ opentoken: code })
    return netClient.httpPost({ url: accountAuthUrl, data: params }).then(res => {
      console.log('vivo auth: ', res)
      let data = res.data
      if (!data || !data.data) {
        return null
      }
      if (data.retcode !== 0) {
        return null
      }
      const openid = data.data.openid
      const unionid = data.data.unionid
      const sessionKey = data.data.session_key
      return { openid, unionid, sessionKey }
    })
  }

  preOrder(platInfo, payData) {
    let data = {
      version: '1.0.0',
      signMethod: 'md5',
      appId: platInfo.appId,
      cpId: platInfo.platSecret,
    }
    Object.assign(payData, data)
    let signStr = PlatVivoNative.generateVivoSign(data, platInfo.platParam2)
    payData.signature = signStr
    console.log(signStr)
    return payData
  }

  checkPayCallback(req, res) {}
}
