import { IPlat, IPlatUser } from './IPlat'
import OAuth from 'wechat-oauth'

/**
 * 微信公众号
 * */
OAuth.prototype.getAccessTokenAsync = function (code) {
  let self = this
  return new Promise((resolve, reject) => {
    self.getAccessToken(code, function (err, result) {
      if (err) {
        return reject && reject(err)
      }
      if (result && result.data) {
        resolve && resolve(result)
      } else {
        reject && reject('no data')
      }
    })
  })
}
OAuth.prototype.getUserAsync = function (options) {
  let self = this
  return new Promise((resolve, reject) => {
    self.getUser(options, function (err, result) {
      if (err) {
        return reject && reject(err)
      }
      if (result && result.data) {
        resolve && resolve(result)
      } else {
        reject && reject('no data')
      }
    })
  })
}
export class PlatWePage implements IPlat {
  platAppId: string
  platSecret: string
  constructor({ platAppId, platSecret }) {
    this.platAppId = platAppId
    this.platSecret = platSecret
  }
  async codeToSession({ code, needUInfo }): Promise<IPlatUser> {
    const client = new OAuth(this.platAppId, this.platSecret)
    client.setOpts({ timeout: 15000 })
    const result = await client.getAccessTokenAsync(code)
    const accessToken = result.data.access_token
    let unionid = ''
    const sessionKey = ''
    const openid = result.data.openid
    needUInfo = needUInfo || false
    let data = { openid, unionid, sessionKey }
    if (needUInfo) {
      const uinfo = await client.getUserAsync(openid)
      Object.assign(data, uinfo)
    }
    return Promise.resolve(data)
  }

  /**
   * 拼接用户授权的 URL
   * @param scope snsapi_base （不弹出授权页面，直接跳转，只能获取用户openid），snsapi_userinfo （弹出授权页面，可通过openid拿到昵称、性别、所在地。并且， 即使在未关注的情况下，只要用户授权，也能获取其信息 ）
   * @param cbUrl
   * @param state 重定向后会带上state参数，开发者可以填写a-zA-Z0-9的参数值，最多128字节
   */
  getAuthorizeURL(scope: string, cbUrl: string, state?: string) {
    const client = new OAuth(this.platAppId, this.platSecret)
    return client.getAuthorizeURL(cbUrl, 'xx', scope)
  }
}
