import { FastifyInstance, FastifyPluginAsync, FastifyReply, FastifyRequest } from 'fastify'
import fastifyPlugin from 'fastify-plugin'
import { Admin } from 'models/admin/Admin'

/**
 * plugin for check token of admin api
 */
declare module 'fastify' {
  interface FastifyRequest {
    jwtVerify: Function
  }
  interface FastifyInstance {
    zAuth: (request: FastifyRequest, reply: FastifyReply) => {}
  }
}

const zAuthPlugin: FastifyPluginAsync = async function (
  fastify: FastifyInstance,
  opts?: { secret: string; expiresIn: string },
) {
  fastify.register(require('fastify-jwt'), {
    secret: opts.secret,
    sign: { expiresIn: opts.expiresIn },
  })

  fastify.decorate('zAuth', async function (request: FastifyRequest, reply: FastifyReply) {
    if (request.permissions || request.roles) {
      try {
        const data = await request.jwtVerify()
        if (!data || !data.id) {
          return reply.send({ code: 10, msg: 'need login' })
        }
        let account = await Admin.findById(data.id)
        if (!account) {
          return reply.send({ code: 10, msg: 'need login' })
        }
        if (!account.roles) {
          return reply.send({ code: 11, msg: 'no permission' })
        }
        let isSupAdmin = account.isSysAdmin()
        if (request.roles && request.roles.indexOf('anon') == -1) {
          if (account.roles && !isSupAdmin) {
            const roles = request.roles.intersect(account.roles)
            if (!roles || roles.length == 0) {
              return reply.send({ code: 12, msg: 'no permission' })
            }
          }
        }
        if (request.permissions && !isSupAdmin) {
          let canDo = false
          for (let role of account.roles) {
            for (let sub of request.permissions) {
              if (this.rbac.can(role, sub[0], sub[1])) {
                canDo = true
                break
              }
              if (canDo) break
            }
          }
          if (!canDo) {
            return reply.send({ code: 11, msg: 'no permission' })
          }
        }

        request.user = account
        return
      } catch (err) {
        // return reply.send({ code: 401, msg: 'need auth' })
        return
      }
    }
  })
}

export default fastifyPlugin(zAuthPlugin, '3.x')
