import * as schedule from 'node-schedule'
import { singleton } from '../decorators/singleton'
import { ShopExam } from '../models/shop/ShopExam'
import { getRankCount, getRankList } from '../services/Rank'
import { rankKey } from '../services/GameLogic'
import { UserMail } from '../models/user/UserMail'
import { Coupon } from '../models/shop/Coupon'
import { Shop } from '../models/shop/Shop'

@singleton
export default class ExamSchedule {
  async parseOneExam(data: any) {
    const shop = data.shop
    let shopName = ''
    const shopData = await Shop.fetchByID(shop)
    if (shopData) {
      shopName = shopData.showName
    }
    const level = data.id + '_total'
    const mode = 2
    const skip = 0
    const key = rankKey(shop, level, mode)
    const limit = (await getRankCount(key)) as number
    let datas: any = await getRankList(skip, limit, key)
    const accountIDS = []
    for (let i = 0, l = datas.length; i < l; i += 2) {
      accountIDS.push(datas[i])
    }
    // accountIDS.push('6001_8005_oPu4N5PiBo4SRWRYaa6byDVcpTfE')
    data.rewardInfo.sort((a, b) => a.rank - b.rank)
    for (let i = 0, l = accountIDS.length; i < l; i++) {
      let rewards = data.getRewardByRank(i + 1)
      let items = []
      for (let r of rewards) {
        let name = '抽奖券'
        if (r.type === 0) {
          let cdata = await Coupon.findById(r.coupon)
          if (cdata) {
            name = cdata.name
          }
        }
        items.push({
          itemId: r.coupon,
          name,
          count: r.count,
        })
      }
      const title = `${data.name} 第${i + 1}名奖励`
      const content = `您在 ${shopName} 的 ${data.name}活动中获得第${i + 1}名, 获得如下奖励`
      await UserMail.addOneMail({
        accountId: accountIDS[i],
        shop,
        items,
        title,
        content,
        senderShop: shopName,
        oid: data.id,
        source: 'exam_rank',
      })
      data.status = 9
      await data.save()
    }
  }
  async parseAllRecord() {
    let records = await ShopExam.allUnSettleRecords()
    for (let record of records) {
      await this.parseOneExam(record)
    }
  }
  scheduleAll() {
    const job = schedule.scheduleJob('1 0 0 * * *', async () => {
      await this.parseAllRecord()
    })
  }
}
