import axios, { AxiosRequestConfig } from 'axios'
import { singleton } from '../decorators/singleton'
import Queue from 'queue'
import { AuditTask } from '../models/AuditTask'
import { ShopPuzzle } from '../models/shop/ShopPuzzle'

export async function auditTxt(txt: string) {
  let data = { content: txt }
  const url = 'http://127.0.0.1:8019/check_all'
  let reqConfig: AxiosRequestConfig = {
    method: 'post',
    url,
    headers: {
      'Cache-Control': 'no-cache',
      'Content-Type': 'application/json',
    },
    data,
  }
  return axios(reqConfig)
}

@singleton
export class AuditSvr {
  private q
  constructor() {
    this.q = new Queue({ concurrency: 1, autostart: true })
  }

  addTask(taskId: string) {
    this.q.push(async function (cb) {
      let taskRecord = await AuditTask.findById(taskId)
      if (!taskRecord) {
        cb()
        return
      }
      let failCount = 0
      let processNum = 0
      let riskNum = 0
      for (let id of taskRecord.ids) {
        let record = await ShopPuzzle.findById(id)
        if (record.status === 1) {
          continue
        }
        processNum++
        let txt = `${record.question},${record.a1},${record.a2},${record.a3},${record.a4}`
        let { data } = await auditTxt(txt)
        record.auditResult = data
        if (data.errcode) {
          failCount++
        } else {
          if (data.wx || data.bd || data.local) {
            record.status = -1
            riskNum++
          } else {
            record.status = 1
          }
        }
        await record.save()
      }
      taskRecord.processNum = processNum
      taskRecord.failNum = failCount
      await taskRecord.save()
      console.log(`audit task finished: ${taskId}, found ${riskNum} risk`)
      cb()
    })
  }
}
