import axios from 'axios'
import { ZError } from '../common/ZError'

/**
 * 查询接受邀请的人数(多店)
 */
export async function getInviteeNum(accountId: string, sessionId: string, shops: number[]) {
  const url = 'https://service.kingsome.cn/webapp/index.php?c=AchievementShare&a=getInviteeNum'
  let _str = shops.join(',')
  let params = {
    session_id: sessionId,
    account_id: accountId,
    achievement_ids: _str,
  }
  return axios.get(url, { params }).then(res => {
    const data = res.data
    if (data.errcode) {
      throw new ZError(data.errcode, data.errmsg)
    }
    return data.invitee_nums
  })
}

/**
 * 查询接受邀请的人数(单店)
 * 对上面多店查询接口的封装
 * @param {string} accountId
 * @param {string} sessionId
 * @param {number} shop
 */
export async function getShopInviteeNum(accountId: string, sessionId: string, shop: number) {
  let records = await getInviteeNum(accountId, sessionId, [shop])
  let num = 0
  if (records?.length > 0) {
    for (let obj of records) {
      if (obj.hasOwnProperty(shop)) {
        num = obj[shop]
        break
      }
    }
  }
  return num
}

/**
 * 查询接受邀请的好友信息数
 * @return {Promise<void>}
 */
export async function getInviteeList(accountId: string, sessionId: string, shops: number[]) {
  const url = 'https://service.kingsome.cn/webapp/index.php?c=AchievementShare&a=getInviteeList'
  let _str = shops.join(',')
  let params = {
    session_id: sessionId,
    account_id: accountId,
    achievement_ids: _str,
  }
  return axios.get(url, { params }).then(res => {
    const data = res.data
    if (data.errcode) {
      throw new ZError(data.errcode, data.errmsg)
    }
    return data.invitee_list
  })
}
