import { singleton } from '../decorators/singleton'

/**
 * 每当有一个人加入一个店铺的游戏, 则设置一个值, 防止重复创建新的房间
 */
@singleton
export class RoomState {
  roomSet: Set<string> = new Set()

  public lock(shop: string) {
    if (this.roomSet.has(shop)) {
      return false
    }
    this.roomSet.add(shop)
    return true
  }

  public isLock(shop: string) {
    return this.roomSet.has(shop)
  }

  public unlock(shop: string) {
    this.roomSet.delete(shop)
  }
}
