import axios, { AxiosRequestConfig } from 'axios'
import fs from 'fs'
import { WxTokenCache } from './WxTokenCache'

export async function generateQr({ appId, appSecret, scene, file }) {
  const stream = fs.createWriteStream(file)
  const token = await refreshToken(appId, appSecret)
  const url = `https://api.weixin.qq.com/wxa/getwxacodeunlimit?access_token=${token}`
  const reqParams = {
    scene: scene,
    width: 430,
    auto_color: false,
    line_color: { r: '0', g: '0', b: '0' },
  }
  let reqConfig: AxiosRequestConfig = {
    method: 'post',
    url,
    headers: {
      'Cache-Control': 'no-cache',
      'Content-Type': 'application/json',
    },
    responseType: 'stream',
    data: reqParams,
  }
  const { data } = await axios(reqConfig)
  data.pipe(stream)
  return true
}

/**
 * 刷新微信api的access token
 * @param {string} appId
 * @param {string} appSecret
 * @param {boolean} refresh 是否强制刷新
 * @return {Promise<any>}
 */
export async function refreshToken(appId: string, appSecret: string, refresh: boolean = false) {
  const util = new WxTokenCache()
  let token = util.getToken(appId)
  if (token && !refresh) {
    return token
  }
  const link = `https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=${appId}&secret=${appSecret}`
  const { data } = await axios.get(link)
  if (!data.errcode) {
    util.updateToken(appId, data.access_token, data.expires_in)
    return data.access_token
  } else {
    throw new Error(data.errmsg)
  }
}

/**
 * 调用微信的security.msgSecCheck接口检查违法违规内容
 * @param {string} content
 * @return {AxiosPromise<any>}
 */
export async function msgSecCheck(content: string) {
  let data = { content }
  const token = await refreshToken('wxf8c3da4e7dfe00a2', '8c0a1e88a6b43e4be80ed6a597c0b047')
  const url = `https://api.weixin.qq.com/wxa/msg_sec_check?access_token=${token}`
  let reqConfig: AxiosRequestConfig = {
    method: 'post',
    url,
    headers: {
      'Cache-Control': 'no-cache',
      'Content-Type': 'application/json',
    },
    data,
  }
  return axios(reqConfig)
}
