import { recoverTypedSignature, SignTypedDataVersion } from '@metamask/eth-sig-util'

export function recoverTypedSignatureV4(signObj: any, signature: string) {
  return recoverTypedSignature({
    data: signObj,
    signature,
    version: SignTypedDataVersion.V4,
  })
}

export function formatAddress(address: string) {
  if (address.length >= 10) {
    return address.substring(0, 8) + '...' + address.substring(address.length - 8)
  } else if (address.length > 0 && address.length < 10) {
    return address
  } else {
    return ''
  }
}

export function buildLoginSignMsg(nonce: string, tips: string) {
  const signMsg = {
    tips,
    nonce,
  }
  const signObj = {
    types: {
      EIP712Domain: [
        { name: 'name', type: 'string' },
        { name: 'version', type: 'string' },
      ],
      set: [
        { name: 'tips', type: 'string' },
        { name: 'nonce', type: 'string' },
      ],
    },
    primaryType: 'set',
    domain: {
      name: 'Auth',
      version: '1',
    },
    message: signMsg,
  }
  return signObj
}
